/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <stdlib.h>
#include <malloc.h>
#include <libspe2.h>
#include <pthread.h>
#include "mars/mars.h"
#include "host_mpu_common.h"

extern struct spe_program_handle task_spe_prog;

static struct mars_context mars;
static struct mars_task_id task_id;
static struct mars_task_params task_params;
static struct mars_task_args task_args;

int main(int argc, char** argv)
{
	int ret;
	char *buffer;

	buffer = (char*)(uint64_t)memalign(128,QUEUE_ENTRY_SIZE);

	printf("Allocated buffer at address %p with size %i\n",buffer,(int)QUEUE_ENTRY_SIZE);

	ret = mars_initialize(&mars, NULL);
	if (ret) {
		printf("MARS context initialize failed! (%d)\n", ret);
		return 1;
	}

	task_params.name = "MPU Task1";
	task_params.elf_image = task_spe_prog.elf_image;
	task_params.context_save_size = 0;
	ret = mars_task_initialize(&mars, &task_id, &task_params);
	if (ret) {
		printf("MARS task initialize failed! (%d)\n", ret);
		return 1;
	}

	task_args.type.u64[0] = (uint64_t)buffer;
	ret = mars_task_schedule(&task_id, &task_args, 0);
	if (ret) {
		printf("MARS task schedule failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_task_wait(&task_id);
	if (ret) {
		printf("MARS task wait failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_task_finalize(&task_id);
	if (ret) {
		printf("MARS task finalize failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_finalize(&mars);
	if (ret) {
		printf("MARS context finalize failed! (%d)\n", ret);
		return 1;
	}

	buffer[QUEUE_ENTRY_SIZE-1] = 0;
	printf("Buffer content: %s\n",buffer);

	free(buffer);

	return 0;
}


