/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <stdio.h>
#include "mars/mars.h"
#include "host_mpu_common.h"
#include <spu_mfcio.h>

static inline void mars_dma_wait(uint32_t tag)
{
#if MARS_DEBUG_ENABLE_DMA_VERBOSE
	MARS_PRINTF("tag=%d\n", tag);
#endif
#if MARS_DEBUG_ENABLE_DMA_ASSERTS
	MARS_ASSERT(tag < 32);			 /* tag 0~31 */
#endif
printf("MPU(%d): mars_dma_wait(tag = %i) before mfc_write_tag_mask(1 << tag);\n",mars_task_get_kernel_id(),tag);
	mfc_write_tag_mask(1 << tag);
printf("MPU(%d): mars_dma_wait(tag = %i) before mfc_write_tag_update_all();\n",mars_task_get_kernel_id(),tag);
	mfc_write_tag_update_all();
printf("MPU(%d): mars_dma_wait(tag = %i) before mfc_read_tag_status();\n",mars_task_get_kernel_id(),tag);
	mfc_read_tag_status();
printf("MPU(%d): mars_dma_wait(tag = %i) returning\n",mars_task_get_kernel_id(),tag);
}

static inline void mars_dma_put(void *ls, uint64_t ea,
					uint32_t size, uint32_t tag)
{
#if MARS_DEBUG_ENABLE_DMA_VERBOSE
	MARS_PRINTF("ls=%p ea=%#llx size=%d tag=%d\n", ls, ea, size, tag);
#endif
#if MARS_DEBUG_ENABLE_DMA_ASSERTS
	MARS_ASSERT(((uintptr_t)ls & 0xf) == 0); /* ls 16 byte alignment */
	MARS_ASSERT(((uintptr_t)ea & 0xf) == 0); /* ea 16 byte alignment */
	MARS_ASSERT((size & 0xf) == 0);		 /* size multiple of 16 bytes */
	MARS_ASSERT(size <= 16384);		 /* size max 16 KB */
	MARS_ASSERT(size);			 /* size not 0 */
	MARS_ASSERT(tag < 32);			 /* tag 0~31 */
#endif
printf("MPU(%d): mars_dma_put(ls = %p, ea = %llx, size = %i, tag = %i);\n",mars_task_get_kernel_id(),ls,ea,size,tag);
	spu_dsync();
	mfc_put(ls, ea, size, tag, 0, 0);
}

static inline void mars_dma_put_and_wait(void *ls, uint64_t ea,
					uint32_t size, uint32_t tag)
{
	mars_dma_put(ls, ea, size, tag);
	mars_dma_wait(tag);
}

struct QUEUE_ENTRY qe_to_push = { "This message is sent from MPU\n" , 0 };

int mars_task_main(const struct mars_task_args *task_args)
{
	(void)task_args;

	uint64_t buffer_ea;

	int tag;

	printf("MPU(%d): %s - Started\n",
		mars_task_get_kernel_id(), mars_task_get_name());

	buffer_ea = task_args->type.u64[0];

	do
	{
		tag = mfc_tag_reserve();
	} while (tag == MFC_TAG_INVALID);

	mars_dma_put_and_wait(&qe_to_push,buffer_ea,QUEUE_ENTRY_SIZE,tag);

	mfc_tag_release(tag);

	printf("MPU(%d): %s - Finished\n",
		mars_task_get_kernel_id(), mars_task_get_name());

	return 0;
}
